@@ -10,39 +10,34 @@ import json
 import os
 import tarfile
 import subprocess
+import typing as t
+
 from contextlib import contextmanager
 from hashlib import sha256
+from urllib.error import URLError
+from urllib.parse import urldefrag
 from shutil import rmtree
 from tempfile import mkdtemp
 
-try:
-    from typing import TYPE_CHECKING
-except ImportError:
-    TYPE_CHECKING = False
-
-if TYPE_CHECKING:
-    from typing import (
-        Any,  # FIXME: !!!111
-        BinaryIO, Dict, IO,
-        Iterator, List, Optional,
-        Set, Tuple, Type, Union,
-    )
-
+if t.TYPE_CHECKING:
     from ansible.galaxy.dependency_resolution.dataclasses import (
-        Candidate, Requirement,
+        Candidate, Collection, Requirement,
     )
     from ansible.galaxy.token import GalaxyToken
 
 from ansible.errors import AnsibleError
 from ansible.galaxy import get_collections_galaxy_meta_info
+from ansible.galaxy.api import should_retry_error
 from ansible.galaxy.dependency_resolution.dataclasses import _GALAXY_YAML
 from ansible.galaxy.user_agent import user_agent
-from ansible.module_utils._text import to_bytes, to_native, to_text
-from ansible.module_utils.six.moves.urllib.error import URLError
-from ansible.module_utils.six.moves.urllib.parse import urldefrag
-from ansible.module_utils.six import raise_from
+from ansible.module_utils.common.text.converters import to_bytes, to_native, to_text
+from ansible.module_utils.api import retry_with_delays_and_condition
+from ansible.module_utils.api import generate_jittered_backoff
+from ansible.module_utils.common.process import get_bin_path
+from ansible.module_utils.common.yaml import yaml_load
 from ansible.module_utils.urls import open_url
 from ansible.utils.display import Display
+from ansible.utils.sentinel import Sentinel
 
 import yaml
 
@@ -61,23 +56,77 @@ class ConcreteArtifactsManager:
         * keeping track of local ones
         * keeping track of Galaxy API tokens for downloads from Galaxy'ish
           as well as the artifact hashes
+        * keeping track of Galaxy API signatures for downloads from Galaxy'ish
         * caching all of above
         * retrieving the metadata out of the downloaded artifacts
     """
-
-    def __init__(self, b_working_directory, validate_certs=True, timeout=60):
-        # type: (bytes, bool, int) -> None
+    def __init__(self, b_working_directory, validate_certs=True, keyring=None, timeout=60, required_signature_count=None, ignore_signature_errors=None):
+        # type: (bytes, bool, str, int, str, list[str]) -> None
         """Initialize ConcreteArtifactsManager caches and costraints."""
         self._validate_certs = validate_certs  # type: bool
-        self._artifact_cache = {}  # type: Dict[bytes, bytes]
-        self._galaxy_artifact_cache = {}  # type: Dict[Union[Candidate, Requirement], bytes]
-        self._artifact_meta_cache = {}  # type: Dict[bytes, Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]]
-        self._galaxy_collection_cache = {}  # type: Dict[Union[Candidate, Requirement], Tuple[str, str, GalaxyToken]]
+        self._artifact_cache = {}  # type: dict[bytes, bytes]
+        self._galaxy_artifact_cache = {}  # type: dict[Candidate | Requirement, bytes]
+        self._artifact_meta_cache = {}  # type: dict[bytes, dict[str, str | list[str] | dict[str, str] | None | t.Type[Sentinel]]]
+        self._galaxy_collection_cache = {}  # type: dict[Candidate | Requirement, tuple[str, str, GalaxyToken]]
+        self._galaxy_collection_origin_cache = {}  # type: dict[Candidate, tuple[str, list[dict[str, str]]]]
         self._b_working_directory = b_working_directory  # type: bytes
+        self._supplemental_signature_cache = {}  # type: dict[str, str]
+        self._keyring = keyring  # type: str
         self.timeout = timeout  # type: int
+        self._required_signature_count = required_signature_count  # type: str
+        self._ignore_signature_errors = ignore_signature_errors  # type: list[str]
+        self._require_build_metadata = True  # type: bool
+
+    @property
+    def keyring(self):
+        return self._keyring
+
+    @property
+    def required_successful_signature_count(self):
+        return self._required_signature_count
+
+    @property
+    def ignore_signature_errors(self):
+        if self._ignore_signature_errors is None:
+            return []
+        return self._ignore_signature_errors
+
+    @property
+    def require_build_metadata(self):
+        # type: () -> bool
+        return self._require_build_metadata
+
+    @require_build_metadata.setter
+    def require_build_metadata(self, value):
+        # type: (bool) -> None
+        self._require_build_metadata = value
+
+    def get_galaxy_artifact_source_info(self, collection):
+        # type: (Candidate) -> dict[str, t.Union[str, list[dict[str, str]]]]
+        server = collection.src.api_server
+
+        try:
+            download_url = self._galaxy_collection_cache[collection][0]
+            signatures_url, signatures = self._galaxy_collection_origin_cache[collection]
+        except KeyError as key_err:
+            raise RuntimeError(
+                'The is no known source for {coll!s}'.
+                format(coll=collection),
+            ) from key_err
+
+        return {
+            "format_version": "1.0.0",
+            "namespace": collection.namespace,
+            "name": collection.name,
+            "version": collection.ver,
+            "server": server,
+            "version_url": signatures_url,
+            "download_url": download_url,
+            "signatures": signatures,
+        }
 
     def get_galaxy_artifact_path(self, collection):
-        # type: (Union[Candidate, Requirement]) -> bytes
+        # type: (t.Union[Candidate, Requirement]) -> bytes
         """Given a Galaxy-stored collection, return a cached path.
 
         If it's not yet on disk, this method downloads the artifact first.
@@ -90,13 +139,10 @@ class ConcreteArtifactsManager:
         try:
             url, sha256_hash, token = self._galaxy_collection_cache[collection]
         except KeyError as key_err:
-            raise_from(
-                RuntimeError(
-                    'The is no known source for {coll!s}'.
-                    format(coll=collection),
-                ),
-                key_err,
-            )
+            raise RuntimeError(
+                'There is no known source for {coll!s}'.
+                format(coll=collection),
+            ) from key_err
 
         display.vvvv(
             "Fetching a collection tarball for '{collection!s}' from "
@@ -112,17 +158,24 @@ class ConcreteArtifactsManager:
                 token=token,
             )  # type: bytes
         except URLError as err:
-            raise_from(
-                AnsibleError(
-                    'Failed to download collection tar '
-                    "from '{coll_src!s}': {download_err!s}".
-                    format(
-                        coll_src=to_native(collection.src),
-                        download_err=to_native(err),
-                    ),
+            raise AnsibleError(
+                'Failed to download collection tar '
+                "from '{coll_src!s}': {download_err!s}".
+                format(
+                    coll_src=to_native(collection.src),
+                    download_err=to_native(err),
                 ),
-                err,
-            )
+            ) from err
+        except Exception as err:
+            raise AnsibleError(
+                'Failed to download collection tar '
+                "from '{coll_src!s}' due to the following unforeseen error: "
+                '{download_err!s}'.
+                format(
+                    coll_src=to_native(collection.src),
+                    download_err=to_native(err),
+                ),
+            ) from err
         else:
             display.vvv(
                 "Collection '{coll!s}' obtained from "
@@ -137,7 +190,7 @@ class ConcreteArtifactsManager:
         return b_artifact_path
 
     def get_artifact_path(self, collection):
-        # type: (Union[Candidate, Requirement]) -> bytes
+        # type: (Collection) -> bytes
         """Given a concrete collection pointer, return a cached path.
 
         If it's not yet on disk, this method downloads the artifact first.
@@ -171,18 +224,15 @@ class ConcreteArtifactsManager:
                     validate_certs=self._validate_certs,
                     timeout=self.timeout
                 )
-            except URLError as err:
-                raise_from(
-                    AnsibleError(
-                        'Failed to download collection tar '
-                        "from '{coll_src!s}': {download_err!s}".
-                        format(
-                            coll_src=to_native(collection.src),
-                            download_err=to_native(err),
-                        ),
+            except Exception as err:
+                raise AnsibleError(
+                    'Failed to download collection tar '
+                    "from '{coll_src!s}': {download_err!s}".
+                    format(
+                        coll_src=to_native(collection.src),
+                        download_err=to_native(err),
                     ),
-                    err,
-                )
+                ) from err
         elif collection.is_scm:
             b_artifact_path = _extract_collection_from_git(
                 collection.src,
@@ -201,16 +251,22 @@ class ConcreteArtifactsManager:
         self._artifact_cache[collection.src] = b_artifact_path
         return b_artifact_path
 
+    def get_artifact_path_from_unknown(self, collection):
+        # type: (Candidate) -> bytes
+        if collection.is_concrete_artifact:
+            return self.get_artifact_path(collection)
+        return self.get_galaxy_artifact_path(collection)
+
     def _get_direct_collection_namespace(self, collection):
-        # type: (Candidate) -> Optional[str]
+        # type: (Candidate) -> t.Optional[str]
         return self.get_direct_collection_meta(collection)['namespace']  # type: ignore[return-value]
 
     def _get_direct_collection_name(self, collection):
-        # type: (Candidate) -> Optional[str]
+        # type: (Collection) -> t.Optional[str]
         return self.get_direct_collection_meta(collection)['name']  # type: ignore[return-value]
 
     def get_direct_collection_fqcn(self, collection):
-        # type: (Candidate) -> Optional[str]
+        # type: (Collection) -> t.Optional[str]
         """Extract FQCN from the given on-disk collection artifact.
 
         If the collection is virtual, ``None`` is returned instead
@@ -226,17 +282,20 @@ class ConcreteArtifactsManager:
         ))
 
     def get_direct_collection_version(self, collection):
-        # type: (Union[Candidate, Requirement]) -> str
+        # type: (Collection) -> str
         """Extract version from the given on-disk collection artifact."""
         return self.get_direct_collection_meta(collection)['version']  # type: ignore[return-value]
 
     def get_direct_collection_dependencies(self, collection):
-        # type: (Union[Candidate, Requirement]) -> Dict[str, str]
+        # type: (t.Union[Candidate, Requirement]) -> dict[str, str]
         """Extract deps from the given on-disk collection artifact."""
-        return self.get_direct_collection_meta(collection)['dependencies']  # type: ignore[return-value]
+        collection_dependencies = self.get_direct_collection_meta(collection)['dependencies']
+        if collection_dependencies is None:
+            collection_dependencies = {}
+        return collection_dependencies  # type: ignore[return-value]
 
     def get_direct_collection_meta(self, collection):
-        # type: (Union[Candidate, Requirement]) -> Dict[str, Optional[Union[str, Dict[str, str], List[str]]]]
+        # type: (Collection) -> dict[str, t.Union[str, dict[str, str], list[str], None, t.Type[Sentinel]]]
         """Extract meta from the given on-disk collection artifact."""
         try:  # FIXME: use unique collection identifier as a cache key?
             return self._artifact_meta_cache[collection.src]
@@ -248,15 +307,12 @@ class ConcreteArtifactsManager:
         elif collection.is_dir:  # should we just build a coll instead?
             # FIXME: what if there's subdirs?
             try:
-                collection_meta = _get_meta_from_dir(b_artifact_path)
+                collection_meta = _get_meta_from_dir(b_artifact_path, self.require_build_metadata)
             except LookupError as lookup_err:
-                raise_from(
-                    AnsibleError(
-                        'Failed to find the collection dir deps: {err!s}'.
-                        format(err=to_native(lookup_err)),
-                    ),
-                    lookup_err,
-                )
+                raise AnsibleError(
+                    'Failed to find the collection dir deps: {err!s}'.
+                    format(err=to_native(lookup_err)),
+                ) from lookup_err
         elif collection.is_scm:
             collection_meta = {
                 'name': None,
@@ -281,22 +337,27 @@ class ConcreteArtifactsManager:
         self._artifact_meta_cache[collection.src] = collection_meta
         return collection_meta
 
-    def save_collection_source(self, collection, url, sha256_hash, token):
-        # type: (Candidate, str, str, GalaxyToken) -> None
+    def save_collection_source(self, collection, url, sha256_hash, token, signatures_url, signatures):
+        # type: (Candidate, str, str, GalaxyToken, str, list[dict[str, str]]) -> None
         """Store collection URL, SHA256 hash and Galaxy API token.
 
         This is a hook that is supposed to be called before attempting to
         download Galaxy-based collections with ``get_galaxy_artifact_path()``.
         """
         self._galaxy_collection_cache[collection] = url, sha256_hash, token
+        self._galaxy_collection_origin_cache[collection] = signatures_url, signatures
 
     @classmethod
     @contextmanager
     def under_tmpdir(
-            cls,  # type: Type[ConcreteArtifactsManager]
+            cls,
             temp_dir_base,  # type: str
             validate_certs=True,  # type: bool
-    ):  # type: (...) -> Iterator[ConcreteArtifactsManager]
+            keyring=None,  # type: str
+            required_signature_count=None,  # type: str
+            ignore_signature_errors=None,  # type: list[str]
+            require_build_metadata=True,  # type: bool
+    ):  # type: (...) -> t.Iterator[ConcreteArtifactsManager]
         """Custom ConcreteArtifactsManager constructor with temp dir.
 
         This method returns a context manager that allocates and cleans
@@ -310,7 +371,13 @@ class ConcreteArtifactsManager:
         )
         b_temp_path = to_bytes(temp_path, errors='surrogate_or_strict')
         try:
-            yield cls(b_temp_path, validate_certs)
+            yield cls(
+                b_temp_path,
+                validate_certs,
+                keyring=keyring,
+                required_signature_count=required_signature_count,
+                ignore_signature_errors=ignore_signature_errors
+            )
         finally:
             rmtree(b_temp_path)
 
@@ -348,34 +415,42 @@ def _extract_collection_from_git(repo_url, coll_ver, b_path):
         dir=b_path,
         prefix=to_bytes(name, errors='surrogate_or_strict'),
     )  # type: bytes
-    git_clone_cmd = 'git', 'clone', git_url, to_text(b_checkout_path)
-    # FIXME: '--depth', '1', '--branch', version
+
+    try:
+        git_executable = get_bin_path('git')
+    except ValueError as err:
+        raise AnsibleError(
+            "Could not find git executable to extract the collection from the Git repository `{repo_url!s}`.".
+            format(repo_url=to_native(git_url))
+        ) from err
+
+    # Perform a shallow clone if simply cloning HEAD
+    if version == 'HEAD':
+        git_clone_cmd = git_executable, 'clone', '--depth=1', git_url, to_text(b_checkout_path)
+    else:
+        git_clone_cmd = git_executable, 'clone', git_url, to_text(b_checkout_path)
+    # FIXME: '--branch', version
+
     try:
         subprocess.check_call(git_clone_cmd)
     except subprocess.CalledProcessError as proc_err:
-        raise_from(
-            AnsibleError(  # should probably be LookupError
-                'Failed to clone a Git repository from `{repo_url!s}`.'.
-                format(repo_url=to_native(git_url)),
-            ),
-            proc_err,
-        )
+        raise AnsibleError(  # should probably be LookupError
+            'Failed to clone a Git repository from `{repo_url!s}`.'.
+            format(repo_url=to_native(git_url)),
+        ) from proc_err
 
-    git_switch_cmd = 'git', 'checkout', to_text(version)
+    git_switch_cmd = git_executable, 'checkout', to_text(version)
     try:
         subprocess.check_call(git_switch_cmd, cwd=b_checkout_path)
     except subprocess.CalledProcessError as proc_err:
-        raise_from(
-            AnsibleError(  # should probably be LookupError
-                'Failed to switch a cloned Git repo `{repo_url!s}` '
-                'to the requested revision `{commitish!s}`.'.
-                format(
-                    commitish=to_native(version),
-                    repo_url=to_native(git_url),
-                ),
+        raise AnsibleError(  # should probably be LookupError
+            'Failed to switch a cloned Git repo `{repo_url!s}` '
+            'to the requested revision `{commitish!s}`.'.
+            format(
+                commitish=to_native(version),
+                repo_url=to_native(git_url),
             ),
-            proc_err,
-        )
+        ) from proc_err
 
     return (
         os.path.join(b_checkout_path, to_bytes(fragment))
@@ -384,8 +459,12 @@ def _extract_collection_from_git(repo_url, coll_ver, b_path):
 
 
 # FIXME: use random subdirs while preserving the file names
+@retry_with_delays_and_condition(
+    backoff_iterator=generate_jittered_backoff(retries=6, delay_base=2, delay_threshold=40),
+    should_retry_error=should_retry_error
+)
 def _download_file(url, b_path, expected_hash, validate_certs, token=None, timeout=60):
-    # type: (str, bytes, Optional[str], bool, GalaxyToken, int) -> bytes
+    # type: (str, bytes, t.Optional[str], bool, GalaxyToken, int) -> bytes
     # ^ NOTE: used in download and verify_collections ^
     b_tarball_name = to_bytes(
         url.rsplit('/', 1)[1], errors='surrogate_or_strict',
@@ -402,15 +481,18 @@ def _download_file(url, b_path, expected_hash, validate_certs, token=None, timeo
     display.display("Downloading %s to %s" % (url, to_text(b_tarball_dir)))
     # NOTE: Galaxy redirects downloads to S3 which rejects the request
     # NOTE: if an Authorization header is attached so don't redirect it
-    resp = open_url(
-        to_native(url, errors='surrogate_or_strict'),
-        validate_certs=validate_certs,
-        headers=None if token is None else token.headers(),
-        unredirected_headers=['Authorization'], http_agent=user_agent(),
-        timeout=timeout
-    )
+    try:
+        resp = open_url(
+            to_native(url, errors='surrogate_or_strict'),
+            validate_certs=validate_certs,
+            headers=None if token is None else token.headers(),
+            unredirected_headers=['Authorization'], http_agent=user_agent(),
+            timeout=timeout
+        )
+    except Exception as err:
+        raise AnsibleError(to_native(err), orig_exc=err)
 
-    with open(b_file_path, 'wb') as download_file:  # type: BinaryIO
+    with open(b_file_path, 'wb') as download_file:  # type: t.BinaryIO
         actual_hash = _consume_file(resp, write_to=download_file)
 
     if expected_hash:
@@ -426,7 +508,7 @@ def _download_file(url, b_path, expected_hash, validate_certs, token=None, timeo
 
 
 def _consume_file(read_from, write_to=None):
-    # type: (BinaryIO, BinaryIO) -> str
+    # type: (t.BinaryIO, t.BinaryIO) -> str
     bufsize = 65536
     sha256_digest = sha256()
     data = read_from.read(bufsize)
@@ -441,19 +523,21 @@ def _consume_file(read_from, write_to=None):
 
 
 def _normalize_galaxy_yml_manifest(
-        galaxy_yml,  # type: Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]
+        galaxy_yml,  # type: dict[str, t.Union[str, list[str], dict[str, str], None, t.Type[Sentinel]]]
         b_galaxy_yml_path,  # type: bytes
+        require_build_metadata=True,  # type: bool
 ):
-    # type: (...) -> Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]
+    # type: (...) -> dict[str, t.Union[str, list[str], dict[str, str], None, t.Type[Sentinel]]]
     galaxy_yml_schema = (
         get_collections_galaxy_meta_info()
-    )  # type: List[Dict[str, Any]]  # FIXME: <--
-    # FIXME: 👆maybe precise type: List[Dict[str, Union[bool, str, List[str]]]]
+    )  # type: list[dict[str, t.Any]]  # FIXME: <--
+    # FIXME: 👆maybe precise type: list[dict[str, t.Union[bool, str, list[str]]]]
 
     mandatory_keys = set()
-    string_keys = set()  # type: Set[str]
-    list_keys = set()  # type: Set[str]
-    dict_keys = set()  # type: Set[str]
+    string_keys = set()  # type: set[str]
+    list_keys = set()  # type: set[str]
+    dict_keys = set()  # type: set[str]
+    sentinel_keys = set()  # type: set[str]
 
     for info in galaxy_yml_schema:
         if info.get('required', False):
@@ -463,16 +547,23 @@ def _normalize_galaxy_yml_manifest(
             'str': string_keys,
             'list': list_keys,
             'dict': dict_keys,
+            'sentinel': sentinel_keys,
         }[info.get('type', 'str')]
         key_list_type.add(info['key'])
 
-    all_keys = frozenset(list(mandatory_keys) + list(string_keys) + list(list_keys) + list(dict_keys))
+    all_keys = frozenset(mandatory_keys | string_keys | list_keys | dict_keys | sentinel_keys)
 
     set_keys = set(galaxy_yml.keys())
     missing_keys = mandatory_keys.difference(set_keys)
     if missing_keys:
-        raise AnsibleError("The collection galaxy.yml at '%s' is missing the following mandatory keys: %s"
-                           % (to_native(b_galaxy_yml_path), ", ".join(sorted(missing_keys))))
+        msg = (
+            "The collection galaxy.yml at '%s' is missing the following mandatory keys: %s"
+            % (to_native(b_galaxy_yml_path), ", ".join(sorted(missing_keys)))
+        )
+        if require_build_metadata:
+            raise AnsibleError(msg)
+        display.warning(msg)
+        raise ValueError(msg)
 
     extra_keys = set_keys.difference(all_keys)
     if len(extra_keys) > 0:
@@ -496,6 +587,10 @@ def _normalize_galaxy_yml_manifest(
         if optional_dict not in galaxy_yml:
             galaxy_yml[optional_dict] = {}
 
+    for optional_sentinel in sentinel_keys:
+        if optional_sentinel not in galaxy_yml:
+            galaxy_yml[optional_sentinel] = Sentinel
+
     # NOTE: `version: null` is only allowed for `galaxy.yml`
     # NOTE: and not `MANIFEST.json`. The use-case for it is collections
     # NOTE: that generate the version from Git before building a
@@ -508,16 +603,18 @@ def _normalize_galaxy_yml_manifest(
 
 def _get_meta_from_dir(
         b_path,  # type: bytes
-):  # type: (...) -> Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]
+        require_build_metadata=True,  # type: bool
+):  # type: (...) -> dict[str, t.Union[str, list[str], dict[str, str], None, t.Type[Sentinel]]]
     try:
         return _get_meta_from_installed_dir(b_path)
     except LookupError:
-        return _get_meta_from_src_dir(b_path)
+        return _get_meta_from_src_dir(b_path, require_build_metadata)
 
 
 def _get_meta_from_src_dir(
         b_path,  # type: bytes
-):  # type: (...) -> Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]
+        require_build_metadata=True,  # type: bool
+):  # type: (...) -> dict[str, t.Union[str, list[str], dict[str, str], None, t.Type[Sentinel]]]
     galaxy_yml = os.path.join(b_path, _GALAXY_YAML)
     if not os.path.isfile(galaxy_yml):
         raise LookupError(
@@ -527,27 +624,31 @@ def _get_meta_from_src_dir(
 
     with open(galaxy_yml, 'rb') as manifest_file_obj:
         try:
-            manifest = yaml.safe_load(manifest_file_obj)
+            manifest = yaml_load(manifest_file_obj)
         except yaml.error.YAMLError as yaml_err:
-            raise_from(
-                AnsibleError(
-                    "Failed to parse the galaxy.yml at '{path!s}' with "
-                    'the following error:\n{err_txt!s}'.
-                    format(
-                        path=to_native(galaxy_yml),
-                        err_txt=to_native(yaml_err),
-                    ),
+            raise AnsibleError(
+                "Failed to parse the galaxy.yml at '{path!s}' with "
+                'the following error:\n{err_txt!s}'.
+                format(
+                    path=to_native(galaxy_yml),
+                    err_txt=to_native(yaml_err),
                 ),
-                yaml_err,
-            )
+            ) from yaml_err
 
-    return _normalize_galaxy_yml_manifest(manifest, galaxy_yml)
+    if not isinstance(manifest, dict):
+        if require_build_metadata:
+            raise AnsibleError(f"The collection galaxy.yml at '{to_native(galaxy_yml)}' is incorrectly formatted.")
+        # Valid build metadata is not required by ansible-galaxy list. Raise ValueError to fall back to implicit metadata.
+        display.warning(f"The collection galaxy.yml at '{to_native(galaxy_yml)}' is incorrectly formatted.")
+        raise ValueError(f"The collection galaxy.yml at '{to_native(galaxy_yml)}' is incorrectly formatted.")
+
+    return _normalize_galaxy_yml_manifest(manifest, galaxy_yml, require_build_metadata)
 
 
 def _get_json_from_installed_dir(
         b_path,  # type: bytes
         filename,  # type: str
-):  # type: (...) -> Dict
+):  # type: (...) -> dict
 
     b_json_filepath = os.path.join(b_path, to_bytes(filename, errors='surrogate_or_strict'))
 
@@ -579,7 +680,7 @@ def _get_json_from_installed_dir(
 
 def _get_meta_from_installed_dir(
         b_path,  # type: bytes
-):  # type: (...) -> Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]
+):  # type: (...) -> dict[str, t.Union[str, list[str], dict[str, str], None, t.Type[Sentinel]]]
     manifest = _get_json_from_installed_dir(b_path, MANIFEST_FILENAME)
     collection_info = manifest['collection_info']
 
@@ -600,7 +701,12 @@ def _get_meta_from_installed_dir(
 
 def _get_meta_from_tar(
         b_path,  # type: bytes
-):  # type: (...) -> Dict[str, Optional[Union[str, List[str], Dict[str, str]]]]
+):  # type: (...) -> dict[str, t.Union[str, list[str], dict[str, str], None, t.Type[Sentinel]]]
+    if not os.path.exists(b_path):
+        raise AnsibleError(
+            f"Unable to find collection artifact file at '{to_native(b_path)}'."
+        )
+
     if not tarfile.is_tarfile(b_path):
         raise AnsibleError(
             "Collection artifact at '{path!s}' is not a valid tar file.".
@@ -648,7 +754,7 @@ def _tarfile_extract(
         tar,  # type: tarfile.TarFile
         member,  # type: tarfile.TarInfo
 ):
-    # type: (...) -> Iterator[Tuple[tarfile.TarInfo, Optional[IO[bytes]]]]
+    # type: (...) -> t.Iterator[tuple[tarfile.TarInfo, t.Optional[t.IO[bytes]]]]
     tar_obj = tar.extractfile(member)
     try:
         yield member, tar_obj
