@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 
 # Copyright 2015 Cristian van Ee <cristian at cvee.org>
@@ -19,31 +18,40 @@ short_description: Manages packages with the I(dnf) package manager
 description:
      - Installs, upgrade, removes, and lists packages and groups with the I(dnf) package manager.
 options:
+  use_backend:
+    description:
+      - By default, this module will select the backend based on the C(ansible_pkg_mgr) fact.
+    default: "auto"
+    choices: [ auto, dnf4, dnf5 ]
+    type: str
+    version_added: 2.15
   name:
     description:
       - "A package name or package specifier with version, like C(name-1.0).
         When using state=latest, this can be '*' which means run: dnf -y update.
-        You can also pass a url or a local path to a rpm file.
+        You can also pass a url or a local path to an rpm file.
         To operate on several packages this can accept a comma separated string of packages or a list of packages."
-      - Comparison operators for package version are valid here C(>), C(<), C(>=), C(<=). Example - C(name>=1.0)
+      - Comparison operators for package version are valid here C(>), C(<), C(>=), C(<=). Example - C(name >= 1.0).
+        Spaces around the operator are required.
       - You can also pass an absolute path for a binary which is provided by the package to install.
         See examples for more information.
-    required: true
     aliases:
         - pkg
     type: list
     elements: str
+    default: []
 
   list:
     description:
-      - Various (non-idempotent) commands for usage with C(/usr/bin/ansible) and I(not) playbooks. See examples.
+      - Various (non-idempotent) commands for usage with C(/usr/bin/ansible) and I(not) playbooks.
+        Use M(ansible.builtin.package_facts) instead of the O(list) argument as a best practice.
     type: str
 
   state:
     description:
-      - Whether to install (C(present), C(latest)), or remove (C(absent)) a package.
-      - Default is C(None), however in effect the default action is C(present) unless the C(autoremove) option is
-        enabled for this module, then C(absent) is inferred.
+      - Whether to install (V(present), V(latest)), or remove (V(absent)) a package.
+      - Default is V(None), however in effect the default action is V(present) unless the O(autoremove) option is
+        enabled for this module, then V(absent) is inferred.
     choices: ['absent', 'present', 'installed', 'removed', 'latest']
     type: str
 
@@ -54,6 +62,7 @@ options:
         When specifying multiple repos, separate them with a ",".
     type: list
     elements: str
+    default: []
 
   disablerepo:
     description:
@@ -62,6 +71,7 @@ options:
         When specifying multiple repos, separate them with a ",".
     type: list
     elements: str
+    default: []
 
   conf_file:
     description:
@@ -71,7 +81,7 @@ options:
   disable_gpg_check:
     description:
       - Whether to disable the GPG checking of signatures of packages being
-        installed. Has an effect only if state is I(present) or I(latest).
+        installed. Has an effect only if O(state) is V(present) or V(latest).
       - This setting affects packages installed from a repository as well as
         "local" packages installed from the filesystem or a URL.
     type: bool
@@ -94,9 +104,9 @@ options:
 
   autoremove:
     description:
-      - If C(yes), removes all "leaf" packages from the system that were originally
+      - If V(true), removes all "leaf" packages from the system that were originally
         installed as dependencies of user-installed packages but which are no longer
-        required by any such package. Should be used alone or when state is I(absent)
+        required by any such package. Should be used alone or when O(state) is V(absent)
     type: bool
     default: "no"
     version_added: "2.4"
@@ -107,6 +117,7 @@ options:
     version_added: "2.7"
     type: list
     elements: str
+    default: []
   skip_broken:
     description:
       - Skip all unavailable packages or packages with broken dependencies
@@ -117,7 +128,7 @@ options:
   update_cache:
     description:
       - Force dnf to check if cache is out of date and redownload if needed.
-        Has an effect only if state is I(present) or I(latest).
+        Has an effect only if O(state) is V(present) or V(latest).
     type: bool
     default: "no"
     aliases: [ expire-cache ]
@@ -125,21 +136,21 @@ options:
   update_only:
     description:
       - When using latest, only update installed packages. Do not install packages.
-      - Has an effect only if state is I(latest)
+      - Has an effect only if O(state) is V(latest)
     default: "no"
     type: bool
     version_added: "2.7"
   security:
     description:
-      - If set to C(yes), and C(state=latest) then only installs updates that have been marked security related.
-      - Note that, similar to ``dnf upgrade-minimal``, this filter applies to dependencies as well.
+      - If set to V(true), and O(state=latest) then only installs updates that have been marked security related.
+      - Note that, similar to C(dnf upgrade-minimal), this filter applies to dependencies as well.
     type: bool
     default: "no"
     version_added: "2.7"
   bugfix:
     description:
-      - If set to C(yes), and C(state=latest) then only installs updates that have been marked bugfix related.
-      - Note that, similar to ``dnf upgrade-minimal``, this filter applies to dependencies as well.
+      - If set to V(true), and O(state=latest) then only installs updates that have been marked bugfix related.
+      - Note that, similar to C(dnf upgrade-minimal), this filter applies to dependencies as well.
     default: "no"
     type: bool
     version_added: "2.7"
@@ -150,28 +161,37 @@ options:
     version_added: "2.7"
     type: list
     elements: str
+    default: []
   disable_plugin:
     description:
       - I(Plugin) name to disable for the install/update operation.
         The disabled plugins will not persist beyond the transaction.
     version_added: "2.7"
     type: list
+    default: []
     elements: str
   disable_excludes:
     description:
       - Disable the excludes defined in DNF config files.
-      - If set to C(all), disables all excludes.
-      - If set to C(main), disable excludes defined in [main] in dnf.conf.
-      - If set to C(repoid), disable excludes defined for given repo id.
+      - If set to V(all), disables all excludes.
+      - If set to V(main), disable excludes defined in [main] in dnf.conf.
+      - If set to V(repoid), disable excludes defined for given repo id.
     version_added: "2.7"
     type: str
   validate_certs:
     description:
-      - This only applies if using a https url as the source of the rpm. e.g. for localinstall. If set to C(no), the SSL certificates will not be validated.
-      - This should only set to C(no) used on personally controlled sites using self-signed certificates as it avoids verifying the source site.
+      - This only applies if using a https url as the source of the rpm. e.g. for localinstall. If set to V(false), the SSL certificates will not be validated.
+      - This should only set to V(false) used on personally controlled sites using self-signed certificates as it avoids verifying the source site.
     type: bool
     default: "yes"
     version_added: "2.7"
+  sslverify:
+    description:
+      - Disables SSL validation of the repository server for this transaction.
+      - This should be set to V(false) if one of the configured repositories is using an untrusted or self-signed certificate.
+    type: bool
+    default: "yes"
+    version_added: "2.13"
   allow_downgrade:
     description:
       - Specify if the named package and version is allowed to downgrade
@@ -188,7 +208,7 @@ options:
   install_repoquery:
     description:
       - This is effectively a no-op in DNF as it is not needed with DNF, but is an accepted parameter for feature
-        parity/compatibility with the I(yum) module.
+        parity/compatibility with the M(ansible.builtin.yum) module.
     type: bool
     default: "yes"
     version_added: "2.7"
@@ -214,12 +234,12 @@ options:
   download_dir:
     description:
       - Specifies an alternate directory to store packages.
-      - Has an effect only if I(download_only) is specified.
+      - Has an effect only if O(download_only) is specified.
     type: str
     version_added: "2.8"
   allowerasing:
     description:
-      - If C(yes) it allows  erasing  of  installed  packages to resolve dependencies.
+      - If V(true) it allows  erasing  of  installed  packages to resolve dependencies.
     required: false
     type: bool
     default: "no"
@@ -255,7 +275,7 @@ attributes:
     platform:
         platforms: rhel
 notes:
-  - When used with a `loop:` each package will be processed individually, it is much more efficient to pass the list directly to the `name` option.
+  - When used with a C(loop:) each package will be processed individually, it is much more efficient to pass the list directly to the I(name) option.
   - Group removal doesn't work if the group was installed with Ansible because
     upstream dnf's API doesn't properly mark groups as installed, therefore upon
     removal the module is unable to detect that the group is installed
@@ -273,80 +293,88 @@ author:
 
 EXAMPLES = '''
 - name: Install the latest version of Apache
-  dnf:
+  ansible.builtin.dnf:
     name: httpd
     state: latest
 
 - name: Install Apache >= 2.4
-  dnf:
-    name: httpd>=2.4
+  ansible.builtin.dnf:
+    name: httpd >= 2.4
     state: present
 
 - name: Install the latest version of Apache and MariaDB
-  dnf:
+  ansible.builtin.dnf:
     name:
       - httpd
       - mariadb-server
     state: latest
 
 - name: Remove the Apache package
-  dnf:
+  ansible.builtin.dnf:
     name: httpd
     state: absent
 
 - name: Install the latest version of Apache from the testing repo
-  dnf:
+  ansible.builtin.dnf:
     name: httpd
     enablerepo: testing
     state: present
 
 - name: Upgrade all packages
-  dnf:
+  ansible.builtin.dnf:
     name: "*"
     state: latest
 
+- name: Update the webserver, depending on which is installed on the system. Do not install the other one
+  ansible.builtin.dnf:
+    name:
+      - httpd
+      - nginx
+    state: latest
+    update_only: yes
+
 - name: Install the nginx rpm from a remote repo
-  dnf:
+  ansible.builtin.dnf:
     name: 'http://nginx.org/packages/centos/6/noarch/RPMS/nginx-release-centos-6-0.el6.ngx.noarch.rpm'
     state: present
 
 - name: Install nginx rpm from a local file
-  dnf:
+  ansible.builtin.dnf:
     name: /usr/local/src/nginx-release-centos-6-0.el6.ngx.noarch.rpm
     state: present
 
 - name: Install Package based upon the file it provides
-  dnf:
+  ansible.builtin.dnf:
     name: /usr/bin/cowsay
     state: present
 
 - name: Install the 'Development tools' package group
-  dnf:
+  ansible.builtin.dnf:
     name: '@Development tools'
     state: present
 
 - name: Autoremove unneeded packages installed as dependencies
-  dnf:
+  ansible.builtin.dnf:
     autoremove: yes
 
 - name: Uninstall httpd but keep its dependencies
-  dnf:
+  ansible.builtin.dnf:
     name: httpd
     state: absent
     autoremove: no
 
 - name: Install a modularity appstream with defined stream and profile
-  dnf:
+  ansible.builtin.dnf:
     name: '@postgresql:9.6/client'
     state: present
 
 - name: Install a modularity appstream with defined stream
-  dnf:
+  ansible.builtin.dnf:
     name: '@postgresql:9.6'
     state: present
 
 - name: Install a modularity appstream with defined profile
-  dnf:
+  ansible.builtin.dnf:
     name: '@postgresql/client'
     state: present
 '''
@@ -355,9 +383,8 @@ import os
 import re
 import sys
 
-from ansible.module_utils._text import to_native, to_text
+from ansible.module_utils.common.text.converters import to_native, to_text
 from ansible.module_utils.urls import fetch_file
-from ansible.module_utils.six import PY2, text_type
 from ansible.module_utils.compat.version import LooseVersion
 
 from ansible.module_utils.basic import AnsibleModule
@@ -540,14 +567,13 @@ class DnfModule(YumDnf):
             e2 = str(e2)
         v2 = str(v2)
         r2 = str(r2)
-        # print '%s, %s, %s vs %s, %s, %s' % (e1, v1, r1, e2, v2, r2)
         rc = dnf.rpm.rpm.labelCompare((e1, v1, r1), (e2, v2, r2))
-        # print '%s, %s, %s vs %s, %s, %s = %s' % (e1, v1, r1, e2, v2, r2, rc)
         return rc
 
     def _ensure_dnf(self):
         locale = get_best_parsable_locale(self.module)
-        os.environ['LC_ALL'] = os.environ['LC_MESSAGES'] = os.environ['LANG'] = locale
+        os.environ['LC_ALL'] = os.environ['LC_MESSAGES'] = locale
+        os.environ['LANGUAGE'] = os.environ['LANG'] = locale
 
         global dnf
         try:
@@ -555,6 +581,7 @@ class DnfModule(YumDnf):
             import dnf.cli
             import dnf.const
             import dnf.exceptions
+            import dnf.package
             import dnf.subject
             import dnf.util
             HAS_DNF = True
@@ -587,7 +614,7 @@ class DnfModule(YumDnf):
             results=[]
         )
 
-    def _configure_base(self, base, conf_file, disable_gpg_check, installroot='/'):
+    def _configure_base(self, base, conf_file, disable_gpg_check, installroot='/', sslverify=True):
         """Configure the dnf Base object."""
 
         conf = base.conf
@@ -616,6 +643,9 @@ class DnfModule(YumDnf):
         # Don't prompt for user confirmations
         conf.assumeyes = True
 
+        # Set certificate validation
+        conf.sslverify = sslverify
+
         # Set installroot
         conf.installroot = installroot
 
@@ -647,6 +677,14 @@ class DnfModule(YumDnf):
         if self.releasever is not None:
             conf.substitutions['releasever'] = self.releasever
 
+        if conf.substitutions.get('releasever') is None:
+            self.module.warn(
+                'Unable to detect release version (use "releasever" option to specify release version)'
+            )
+            # values of conf.substitutions are expected to be strings
+            # setting this to an empty string instead of None appears to mimic the DNF CLI behavior
+            conf.substitutions['releasever'] = ''
+
         # Set skip_broken (in dnf this is strict=0)
         if self.skip_broken:
             conf.strict = 0
@@ -686,10 +724,10 @@ class DnfModule(YumDnf):
                 for repo in repos.get_matching(repo_pattern):
                     repo.enable()
 
-    def _base(self, conf_file, disable_gpg_check, disablerepo, enablerepo, installroot):
+    def _base(self, conf_file, disable_gpg_check, disablerepo, enablerepo, installroot, sslverify):
         """Return a fully configured dnf Base object."""
         base = dnf.Base()
-        self._configure_base(base, conf_file, disable_gpg_check, installroot)
+        self._configure_base(base, conf_file, disable_gpg_check, installroot, sslverify)
         try:
             # this method has been supported in dnf-4.2.17-6 or later
             # https://bugzilla.redhat.com/show_bug.cgi?id=1788212
@@ -785,10 +823,7 @@ class DnfModule(YumDnf):
         else:
             package_spec['name'] = name
 
-        if installed.filter(**package_spec):
-            return True
-        else:
-            return False
+        return bool(installed.filter(**package_spec))
 
     def _is_newer_version_installed(self, pkg_name):
         candidate_pkg = self._packagename_dict(pkg_name)
@@ -808,13 +843,8 @@ class DnfModule(YumDnf):
                 candidate_pkg['epoch'], candidate_pkg['version'], candidate_pkg['release'],
             )
 
-            if evr_cmp == 1:
-                return True
-            else:
-                return False
-
+            return evr_cmp == 1
         else:
-
             return False
 
     def _mark_package_install(self, pkg_spec, upgrade=False):
@@ -829,21 +859,31 @@ class DnfModule(YumDnf):
                     # on a system's package set (pending the yum repo has many old
                     # NVRs indexed)
                     if upgrade:
-                        if is_installed:
+                        if is_installed:  # Case 1
+                            # TODO: Is this case reachable?
+                            #
+                            # _is_installed() demands a name (*not* NVR) or else is always False
+                            # (wildcards are treated literally).
+                            #
+                            # Meanwhile, _is_newer_version_installed() demands something versioned
+                            # or else is always false.
+                            #
+                            # I fail to see how they can both be true at the same time for any
+                            # given pkg_spec. -re
                             self.base.upgrade(pkg_spec)
-                        else:
-                            self.base.install(pkg_spec)
-                    else:
-                        self.base.install(pkg_spec)
-                else:  # Nothing to do, report back
+                        else:  # Case 2
+                            self.base.install(pkg_spec, strict=self.base.conf.strict)
+                    else:  # Case 3
+                        self.base.install(pkg_spec, strict=self.base.conf.strict)
+                else:  # Case 4, Nothing to do, report back
                     pass
-            elif is_installed:  # An potentially older (or same) version is installed
-                if upgrade:
+            elif is_installed:  # A potentially older (or same) version is installed
+                if upgrade:  # Case 5
                     self.base.upgrade(pkg_spec)
-                else:  # Nothing to do, report back
+                else:  # Case 6, Nothing to do, report back
                     pass
-            else:  # The package is not installed, simply install it
-                self.base.install(pkg_spec)
+            else:  # Case 7, The package is not installed, simply install it
+                self.base.install(pkg_spec, strict=self.base.conf.strict)
 
             return {'failed': False, 'msg': '', 'failure': '', 'rc': 0}
 
@@ -926,12 +966,14 @@ class DnfModule(YumDnf):
     def _update_only(self, pkgs):
         not_installed = []
         for pkg in pkgs:
-            if self._is_installed(pkg):
+            if self._is_installed(
+                    self._package_dict(pkg)["nevra"] if isinstance(pkg, dnf.package.Package) else pkg
+            ):
                 try:
-                    if isinstance(to_text(pkg), text_type):
-                        self.base.upgrade(pkg)
-                    else:
+                    if isinstance(pkg, dnf.package.Package):
                         self.base.package_upgrade(pkg)
+                    else:
+                        self.base.upgrade(pkg)
                 except Exception as e:
                     self.module.fail_json(
                         msg="Error occurred attempting update_only operation: {0}".format(to_native(e)),
@@ -965,9 +1007,9 @@ class DnfModule(YumDnf):
                 try:
                     if self._is_newer_version_installed(self._package_dict(pkg)['nevra']):
                         if self.allow_downgrade:
-                            self.base.package_install(pkg)
+                            self.base.package_install(pkg, strict=self.base.conf.strict)
                     else:
-                        self.base.package_install(pkg)
+                        self.base.package_install(pkg, strict=self.base.conf.strict)
                 except Exception as e:
                     self.module.fail_json(
                         msg="Error occurred attempting remote rpm operation: {0}".format(to_native(e)),
@@ -1159,9 +1201,13 @@ class DnfModule(YumDnf):
                         response['results'].append("Packages providing %s not installed due to update_only specified" % spec)
                 else:
                     for pkg_spec in pkg_specs:
-                        # best effort causes to install the latest package
-                        # even if not previously installed
-                        self.base.conf.best = True
+                        # Previously we forced base.conf.best=True here.
+                        # However in 2.11+ there is a self.nobest option, so defer to that.
+                        # Note, however, that just because nobest isn't set, doesn't mean that
+                        # base.conf.best is actually true. We only force it false in
+                        # _configure_base(), we never set it to true, and it can default to false.
+                        # Thus, we still need to explicitly set it here.
+                        self.base.conf.best = not self.nobest
                         install_result = self._mark_package_install(pkg_spec, upgrade=True)
                         if install_result['failed']:
                             if install_result['msg']:
@@ -1241,6 +1287,15 @@ class DnfModule(YumDnf):
                     self.base.autoremove()
 
         try:
+            # NOTE for people who go down the rabbit hole of figuring out why
+            # resolve() throws DepsolveError here on dep conflict, but not when
+            # called from the CLI: It's controlled by conf.best. When best is
+            # set, Hawkey will fail the goal, and resolve() in dnf.base.Base
+            # will throw. Otherwise if it's not set, the update (install) will
+            # be (almost silently) removed from the goal, and Hawkey will report
+            # success. Note that in this case, similar to the CLI, skip_broken
+            # does nothing to help here, so we don't take it into account at
+            # all.
             if not self.base.resolve(allow_erasing=self.allowerasing):
                 if failure_response['failures']:
                     failure_response['msg'] = 'Failed to install some of the specified packages'
@@ -1298,7 +1353,7 @@ class DnfModule(YumDnf):
                             fail = True
 
                         if fail:
-                            msg = 'Failed to validate GPG signature for {0}'.format(package)
+                            msg = 'Failed to validate GPG signature for {0}: {1}'.format(package, gpgerr)
                             self.module.fail_json(msg)
 
                 if self.download_only:
@@ -1350,7 +1405,7 @@ class DnfModule(YumDnf):
         if self.update_cache and not self.names and not self.list:
             self.base = self._base(
                 self.conf_file, self.disable_gpg_check, self.disablerepo,
-                self.enablerepo, self.installroot
+                self.enablerepo, self.installroot, self.sslverify
             )
             self.module.exit_json(
                 msg="Cache updated",
@@ -1368,7 +1423,7 @@ class DnfModule(YumDnf):
         if self.list:
             self.base = self._base(
                 self.conf_file, self.disable_gpg_check, self.disablerepo,
-                self.enablerepo, self.installroot
+                self.enablerepo, self.installroot, self.sslverify
             )
             self.list_items(self.list)
         else:
@@ -1381,7 +1436,7 @@ class DnfModule(YumDnf):
                 )
             self.base = self._base(
                 self.conf_file, self.disable_gpg_check, self.disablerepo,
-                self.enablerepo, self.installroot
+                self.enablerepo, self.installroot, self.sslverify
             )
 
             if self.with_modules:
@@ -1406,6 +1461,7 @@ def main():
     # backported to yum because yum is now in "maintenance mode" upstream
     yumdnf_argument_spec['argument_spec']['allowerasing'] = dict(default=False, type='bool')
     yumdnf_argument_spec['argument_spec']['nobest'] = dict(default=False, type='bool')
+    yumdnf_argument_spec['argument_spec']['use_backend'] = dict(default='auto', choices=['auto', 'dnf4', 'dnf5'])
 
     module = AnsibleModule(
         **yumdnf_argument_spec
