@@ -4,7 +4,6 @@
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
-from ansible import constants as C
 from ansible.plugins.action import ActionBase
 from ansible.utils.vars import merge_hash
 
@@ -16,10 +15,6 @@ class ActionModule(ActionBase):
         results = super(ActionModule, self).run(tmp, task_vars)
         del tmp  # tmp no longer has any effect
 
-        # Command module has a special config option to turn off the command nanny warnings
-        if 'warn' not in self._task.args and C.COMMAND_WARNINGS:
-            self._task.args['warn'] = C.COMMAND_WARNINGS
-
         wrap_async = self._task.async_val and not self._connection.has_native_async
         # explicitly call `ansible.legacy.command` for backcompat to allow library/ override of `command` while not allowing
         # collections search for an unqualified `command` module
