@@ -77,14 +77,21 @@ RETURN = '''
 #
 '''
 
+import datetime
 import os
+import subprocess
+import tarfile
 import tempfile
 import yaml
 
 from ansible.module_utils.basic import AnsibleModule
-from ansible.module_utils._text import to_bytes
+from ansible.module_utils.common.text.converters import to_bytes
 from functools import partial
 from multiprocessing import dummy as threading
+from multiprocessing import TimeoutError
+
+
+COLLECTIONS_BUILD_AND_PUBLISH_TIMEOUT = 180
 
 
 def publish_collection(module, collection):
@@ -98,6 +105,7 @@ def publish_collection(module, collection):
     collection_dir = os.path.join(module.tmpdir, "%s-%s-%s" % (namespace, name, version))
     b_collection_dir = to_bytes(collection_dir, errors='surrogate_or_strict')
     os.mkdir(b_collection_dir)
+    os.mkdir(os.path.join(b_collection_dir, b'meta'))
 
     with open(os.path.join(b_collection_dir, b'README.md'), mode='wb') as fd:
         fd.write(b"Collection readme")
@@ -114,6 +122,8 @@ def publish_collection(module, collection):
     }
     with open(os.path.join(b_collection_dir, b'galaxy.yml'), mode='wb') as fd:
         fd.write(to_bytes(yaml.safe_dump(galaxy_meta), errors='surrogate_or_strict'))
+    with open(os.path.join(b_collection_dir, b'meta/runtime.yml'), mode='wb') as fd:
+        fd.write(b'requires_ansible: ">=1.0.0"')
 
     with tempfile.NamedTemporaryFile(mode='wb') as temp_fd:
         temp_fd.write(b"data")
@@ -141,6 +151,26 @@ def publish_collection(module, collection):
             'stderr': stderr,
         }
 
+        if module.params['signature_dir'] is not None:
+            # To test user-provided signatures, we need to sign the MANIFEST.json before publishing
+
+            # Extract the tarfile to sign the MANIFEST.json
+            with tarfile.open(collection_path, mode='r') as collection_tar:
+                # deprecated: description='extractall fallback without filter' python_version='3.11'
+                # Replace 'tar_filter' with 'data_filter' and 'filter=tar' with 'filter=data' once Python 3.12 is minimum requirement.
+                if hasattr(tarfile, 'tar_filter'):
+                    collection_tar.extractall(path=os.path.join(collection_dir, '%s-%s-%s' % (namespace, name, version)), filter='tar')
+                else:
+                    collection_tar.extractall(path=os.path.join(collection_dir, '%s-%s-%s' % (namespace, name, version)))
+
+            manifest_path = os.path.join(collection_dir, '%s-%s-%s' % (namespace, name, version), 'MANIFEST.json')
+            signature_path = os.path.join(module.params['signature_dir'], '%s-%s-%s-MANIFEST.json.asc' % (namespace, name, version))
+            sign_manifest(signature_path, manifest_path, module, result)
+
+            # Create the tarfile containing the signed MANIFEST.json
+            with tarfile.open(collection_path, "w:gz") as tar:
+                tar.add(os.path.join(collection_dir, '%s-%s-%s' % (namespace, name, version)), arcname=os.path.sep)
+
     publish_args = ['ansible-galaxy', 'collection', 'publish', collection_path, '--server', module.params['server']]
     if module.params['token']:
         publish_args.extend(['--token', module.params['token']])
@@ -155,6 +185,47 @@ def publish_collection(module, collection):
     return result
 
 
+def sign_manifest(signature_path, manifest_path, module, collection_setup_result):
+    collection_setup_result['gpg_detach_sign'] = {'signature_path': signature_path}
+
+    status_fd_read, status_fd_write = os.pipe()
+    gpg_cmd = [
+        "gpg",
+        "--batch",
+        "--pinentry-mode",
+        "loopback",
+        "--yes",
+        "--homedir",
+        module.params['signature_dir'],
+        "--detach-sign",
+        "--armor",
+        "--output",
+        signature_path,
+        manifest_path,
+    ]
+    try:
+        p = subprocess.Popen(
+            gpg_cmd,
+            stdin=subprocess.PIPE,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+            pass_fds=(status_fd_write,),
+            encoding='utf8',
+        )
+    except (FileNotFoundError, subprocess.SubprocessError) as err:
+        collection_setup_result['gpg_detach_sign']['error'] = "Failed during GnuPG verification with command '{gpg_cmd}': {err}".format(
+            gpg_cmd=gpg_cmd, err=err
+        )
+    else:
+        stdout, stderr = p.communicate()
+        collection_setup_result['gpg_detach_sign']['stdout'] = stdout
+        if stderr:
+            error = "Failed during GnuPG verification with command '{gpg_cmd}':\n{stderr}".format(gpg_cmd=gpg_cmd, stderr=stderr)
+            collection_setup_result['gpg_detach_sign']['error'] = error
+    finally:
+        os.close(status_fd_write)
+
+
 def run_module():
     module_args = dict(
         server=dict(type='str', required=True),
@@ -171,6 +242,7 @@ def run_module():
                 use_symlink=dict(type='bool', default=False),
             ),
         ),
+        signature_dir=dict(type='path', default=None),
     )
 
     module = AnsibleModule(
@@ -178,17 +250,27 @@ def run_module():
         supports_check_mode=False
     )
 
-    result = dict(changed=True, results=[])
+    start = datetime.datetime.now()
+    result = dict(changed=True, results=[], start=str(start))
 
     pool = threading.Pool(4)
     publish_func = partial(publish_collection, module)
-    result['results'] = pool.map(publish_func, module.params['collections'])
+    try:
+        result['results'] = pool.map_async(
+            publish_func, module.params['collections'],
+        ).get(timeout=COLLECTIONS_BUILD_AND_PUBLISH_TIMEOUT)
+    except TimeoutError as timeout_err:
+        module.fail_json(
+            'Timed out waiting for collections to be provisioned.',
+        )
 
     failed = bool(sum(
         r['build']['rc'] + r['publish']['rc'] for r in result['results']
     ))
 
-    module.exit_json(failed=failed, **result)
+    end = datetime.datetime.now()
+    delta = end - start
+    module.exit_json(failed=failed, end=str(end), delta=str(delta), **result)
 
 
 def main():
