@@ -1,9 +1,6 @@
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
-import json
-import sys
-
 from ansible.module_utils.basic import AnsibleModule
 from ansible.module_utils.facts.utils import (
     get_file_content,
