@@ -2,6 +2,7 @@
 from __future__ import annotations
 
 import argparse
+import collections.abc as c
 import dataclasses
 import enum
 import os
@@ -26,8 +27,8 @@ from ..docker_util import (
 )
 
 from ..completion import (
-    DOCKER_COMPLETION,
-    REMOTE_COMPLETION,
+    docker_completion,
+    remote_completion,
     filter_completion,
 )
 
@@ -55,25 +56,25 @@ from ..data import (
 )
 
 
-def filter_python(version, versions):  # type: (t.Optional[str], t.Optional[t.List[str]]) -> t.Optional[str]
+def filter_python(version: t.Optional[str], versions: t.Optional[c.Sequence[str]]) -> t.Optional[str]:
     """If a Python version is given and is in the given version list, return that Python version, otherwise return None."""
     return version if version in versions else None
 
 
-def controller_python(version):  # type: (t.Optional[str]) -> t.Optional[str]
+def controller_python(version: t.Optional[str]) -> t.Optional[str]:
     """If a Python version is given and is supported by the controller, return that Python version, otherwise return None."""
     return filter_python(version, CONTROLLER_PYTHON_VERSIONS)
 
 
-def get_fallback_remote_controller():  # type: () -> str
+def get_fallback_remote_controller() -> str:
     """Return the remote fallback platform for the controller."""
     platform = 'freebsd'  # lower cost than RHEL and macOS
-    candidates = [item for item in filter_completion(REMOTE_COMPLETION).values() if item.controller_supported and item.platform == platform]
+    candidates = [item for item in filter_completion(remote_completion()).values() if item.controller_supported and item.platform == platform]
     fallback = sorted(candidates, key=lambda value: str_to_version(value.version), reverse=True)[0]
     return fallback.name
 
 
-def get_option_name(name):  # type: (str) -> str
+def get_option_name(name: str) -> str:
     """Return a command-line option name from the given option name."""
     if name == 'targets':
         name = 'target'
@@ -83,31 +84,36 @@ def get_option_name(name):  # type: (str) -> str
 
 class PythonVersionUnsupportedError(ApplicationError):
     """A Python version was requested for a context which does not support that version."""
-    def __init__(self, context, version, versions):
+
+    def __init__(self, context: str, version: str, versions: c.Iterable[str]) -> None:
         super().__init__(f'Python {version} is not supported by environment `{context}`. Supported Python version(s) are: {", ".join(versions)}')
 
 
 class PythonVersionUnspecifiedError(ApplicationError):
     """A Python version was not specified for a context which is unknown, thus the Python version is unknown."""
-    def __init__(self, context):
+
+    def __init__(self, context: str) -> None:
         super().__init__(f'A Python version was not specified for environment `{context}`. Use the `--python` option to specify a Python version.')
 
 
 class ControllerNotSupportedError(ApplicationError):
     """Option(s) were specified which do not provide support for the controller and would be ignored because they are irrelevant for the target."""
-    def __init__(self, context):
+
+    def __init__(self, context: str) -> None:
         super().__init__(f'Environment `{context}` does not provide a Python version supported by the controller.')
 
 
 class OptionsConflictError(ApplicationError):
     """Option(s) were specified which conflict with other options."""
-    def __init__(self, first, second):
+
+    def __init__(self, first: c.Iterable[str], second: c.Iterable[str]) -> None:
         super().__init__(f'Options `{" ".join(first)}` cannot be combined with options `{" ".join(second)}`.')
 
 
 @dataclasses.dataclass(frozen=True)
 class LegacyHostOptions:
     """Legacy host options used prior to the availability of separate controller and target host configuration."""
+
     python: t.Optional[str] = None
     python_interpreter: t.Optional[str] = None
     local: t.Optional[bool] = None
@@ -115,18 +121,19 @@ class LegacyHostOptions:
     venv_system_site_packages: t.Optional[bool] = None
     remote: t.Optional[str] = None
     remote_provider: t.Optional[str] = None
+    remote_arch: t.Optional[str] = None
     docker: t.Optional[str] = None
     docker_privileged: t.Optional[bool] = None
     docker_seccomp: t.Optional[str] = None
     docker_memory: t.Optional[int] = None
-    windows: t.Optional[t.List[str]] = None
-    platform: t.Optional[t.List[str]] = None
-    platform_collection: t.Optional[t.List[t.Tuple[str, str]]] = None
-    platform_connection: t.Optional[t.List[t.Tuple[str, str]]] = None
+    windows: t.Optional[list[str]] = None
+    platform: t.Optional[list[str]] = None
+    platform_collection: t.Optional[list[tuple[str, str]]] = None
+    platform_connection: t.Optional[list[tuple[str, str]]] = None
     inventory: t.Optional[str] = None
 
     @staticmethod
-    def create(namespace):  # type: (t.Union[argparse.Namespace, types.SimpleNamespace]) -> LegacyHostOptions
+    def create(namespace: t.Union[argparse.Namespace, types.SimpleNamespace]) -> LegacyHostOptions:
         """Create legacy host options from the given namespace."""
         kwargs = {field.name: getattr(namespace, field.name, None) for field in dataclasses.fields(LegacyHostOptions)}
 
@@ -136,29 +143,30 @@ class LegacyHostOptions:
         return LegacyHostOptions(**kwargs)
 
     @staticmethod
-    def purge_namespace(namespace):  # type: (t.Union[argparse.Namespace, types.SimpleNamespace]) -> None
+    def purge_namespace(namespace: t.Union[argparse.Namespace, types.SimpleNamespace]) -> None:
         """Purge legacy host options fields from the given namespace."""
-        for field in dataclasses.fields(LegacyHostOptions):   # type: dataclasses.Field
+        for field in dataclasses.fields(LegacyHostOptions):
             if hasattr(namespace, field.name):
                 delattr(namespace, field.name)
 
     @staticmethod
-    def purge_args(args):  # type: (t.List[str]) -> t.List[str]
+    def purge_args(args: list[str]) -> list[str]:
         """Purge legacy host options from the given command line arguments."""
-        fields = dataclasses.fields(LegacyHostOptions)  # type: t.Tuple[dataclasses.Field, ...]
-        filters = {get_option_name(field.name): 0 if field.type is t.Optional[bool] else 1 for field in fields}  # type: t.Dict[str, int]
+        fields: tuple[dataclasses.Field, ...] = dataclasses.fields(LegacyHostOptions)
+        filters: dict[str, int] = {get_option_name(field.name): 0 if field.type is t.Optional[bool] else 1 for field in fields}
 
         return filter_args(args, filters)
 
-    def get_options_used(self):  # type: () -> t.Tuple[str, ...]
+    def get_options_used(self) -> tuple[str, ...]:
         """Return a tuple of the command line options used."""
-        fields = dataclasses.fields(self)  # type: t.Tuple[dataclasses.Field, ...]
+        fields: tuple[dataclasses.Field, ...] = dataclasses.fields(self)
         options = tuple(sorted(get_option_name(field.name) for field in fields if getattr(self, field.name)))
         return options
 
 
 class TargetMode(enum.Enum):
     """Type of provisioning to use for the targets."""
+
     WINDOWS_INTEGRATION = enum.auto()  # windows-integration
     NETWORK_INTEGRATION = enum.auto()  # network-integration
     POSIX_INTEGRATION = enum.auto()  # integration
@@ -168,31 +176,31 @@ class TargetMode(enum.Enum):
     NO_TARGETS = enum.auto()  # coverage
 
     @property
-    def one_host(self):
+    def one_host(self) -> bool:
         """Return True if only one host (the controller) should be used, otherwise return False."""
         return self in (TargetMode.SANITY, TargetMode.UNITS, TargetMode.NO_TARGETS)
 
     @property
-    def no_fallback(self):
+    def no_fallback(self) -> bool:
         """Return True if no fallback is acceptable for the controller (due to options not applying to the target), otherwise return False."""
         return self in (TargetMode.WINDOWS_INTEGRATION, TargetMode.NETWORK_INTEGRATION, TargetMode.NO_TARGETS)
 
     @property
-    def multiple_pythons(self):
+    def multiple_pythons(self) -> bool:
         """Return True if multiple Python versions are allowed, otherwise False."""
         return self in (TargetMode.SANITY, TargetMode.UNITS)
 
     @property
-    def has_python(self):
+    def has_python(self) -> bool:
         """Return True if this mode uses Python, otherwise False."""
         return self in (TargetMode.POSIX_INTEGRATION, TargetMode.SANITY, TargetMode.UNITS, TargetMode.SHELL)
 
 
 def convert_legacy_args(
-        argv,  # type: t.List[str]
-        args,  # type: t.Union[argparse.Namespace, types.SimpleNamespace]
-        mode,  # type: TargetMode
-):  # type: (...) -> HostSettings
+    argv: list[str],
+    args: t.Union[argparse.Namespace, types.SimpleNamespace],
+    mode: TargetMode,
+) -> HostSettings:
     """Convert pre-split host arguments in the given namespace to their split counterparts."""
     old_options = LegacyHostOptions.create(args)
     old_options.purge_namespace(args)
@@ -201,6 +209,9 @@ def convert_legacy_args(
         '--controller',
         '--target',
         '--target-python',
+        '--target-posix',
+        '--target-windows',
+        '--target-network',
     ]
 
     used_old_options = old_options.get_options_used()
@@ -237,8 +248,8 @@ def convert_legacy_args(
     args.targets = targets
 
     if used_default_pythons:
-        targets = t.cast(t.List[ControllerConfig], targets)
-        skipped_python_versions = sorted_versions(list(set(SUPPORTED_PYTHON_VERSIONS) - {target.python.version for target in targets}))
+        control_targets = t.cast(list[ControllerConfig], targets)
+        skipped_python_versions = sorted_versions(list(set(SUPPORTED_PYTHON_VERSIONS) - {target.python.version for target in control_targets}))
     else:
         skipped_python_versions = []
 
@@ -257,13 +268,15 @@ def convert_legacy_args(
 
 
 def controller_targets(
-        mode,  # type: TargetMode
-        options,  # type: LegacyHostOptions
-        controller,  # type: ControllerHostConfig
-):  # type: (...) -> t.List[ControllerConfig]
+    mode: TargetMode,
+    options: LegacyHostOptions,
+    controller: ControllerHostConfig,
+) -> list[HostConfig]:
     """Return the configuration for controller targets."""
     python = native_python(options)
 
+    targets: list[HostConfig]
+
     if python:
         targets = [ControllerConfig(python=python)]
     else:
@@ -272,7 +285,7 @@ def controller_targets(
     return targets
 
 
-def native_python(options):  # type: (LegacyHostOptions) -> t.Optional[NativePythonConfig]
+def native_python(options: LegacyHostOptions) -> t.Optional[NativePythonConfig]:
     """Return a NativePythonConfig for the given version if it is not None, otherwise return None."""
     if not options.python and not options.python_interpreter:
         return None
@@ -281,9 +294,9 @@ def native_python(options):  # type: (LegacyHostOptions) -> t.Optional[NativePyt
 
 
 def get_legacy_host_config(
-        mode,  # type: TargetMode
-        options,  # type: LegacyHostOptions
-):  # type: (...) -> t.Tuple[HostConfig, t.List[HostConfig], t.Optional[FallbackDetail]]
+    mode: TargetMode,
+    options: LegacyHostOptions,
+) -> tuple[ControllerHostConfig, list[HostConfig], t.Optional[FallbackDetail]]:
     """
     Returns controller and target host configs derived from the provided legacy host options.
     The goal is to match the original behavior, by using non-split testing whenever possible.
@@ -294,7 +307,10 @@ def get_legacy_host_config(
     docker_fallback = 'default'
     remote_fallback = get_fallback_remote_controller()
 
-    controller_fallback = None  # type: t.Optional[t.Tuple[str, str, FallbackReason]]
+    controller_fallback: t.Optional[tuple[str, str, FallbackReason]] = None
+
+    controller: t.Optional[ControllerHostConfig]
+    targets: list[HostConfig]
 
     if options.venv:
         if controller_python(options.python) or not options.python:
@@ -304,19 +320,26 @@ def get_legacy_host_config(
             controller = OriginConfig(python=VirtualPythonConfig(version='default', system_site_packages=options.venv_system_site_packages))
 
         if mode in (TargetMode.SANITY, TargetMode.UNITS):
-            targets = controller_targets(mode, options, controller)
+            python = native_python(options)
+
+            if python:
+                control_targets = [ControllerConfig(python=python)]
+            else:
+                control_targets = controller.get_default_targets(HostContext(controller_config=controller))
 
             # Target sanity tests either have no Python requirements or manage their own virtual environments.
-            # Thus there is no point in setting up virtual environments ahead of time for them.
+            # Thus, there is no point in setting up virtual environments ahead of time for them.
 
             if mode == TargetMode.UNITS:
                 targets = [ControllerConfig(python=VirtualPythonConfig(version=target.python.version, path=target.python.path,
-                                                                       system_site_packages=options.venv_system_site_packages)) for target in targets]
+                                                                       system_site_packages=options.venv_system_site_packages)) for target in control_targets]
+            else:
+                targets = t.cast(list[HostConfig], control_targets)
         else:
             targets = [ControllerConfig(python=VirtualPythonConfig(version=options.python or 'default',
                                                                    system_site_packages=options.venv_system_site_packages))]
     elif options.docker:
-        docker_config = filter_completion(DOCKER_COMPLETION).get(options.docker)
+        docker_config = filter_completion(docker_completion()).get(options.docker)
 
         if docker_config:
             if options.python and options.python not in docker_config.supported_pythons:
@@ -350,7 +373,7 @@ def get_legacy_host_config(
                 targets = [DockerConfig(name=options.docker, python=native_python(options),
                                         privileged=options.docker_privileged, seccomp=options.docker_seccomp, memory=options.docker_memory)]
     elif options.remote:
-        remote_config = filter_completion(REMOTE_COMPLETION).get(options.remote)
+        remote_config = filter_completion(remote_completion()).get(options.remote)
         context, reason = None, None
 
         if remote_config:
@@ -359,33 +382,34 @@ def get_legacy_host_config(
 
             if remote_config.controller_supported:
                 if controller_python(options.python) or not options.python:
-                    controller = PosixRemoteConfig(name=options.remote, python=native_python(options), provider=options.remote_provider)
+                    controller = PosixRemoteConfig(name=options.remote, python=native_python(options), provider=options.remote_provider,
+                                                   arch=options.remote_arch)
                     targets = controller_targets(mode, options, controller)
                 else:
                     controller_fallback = f'remote:{options.remote}', f'--remote {options.remote} --python {options.python}', FallbackReason.PYTHON
-                    controller = PosixRemoteConfig(name=options.remote, provider=options.remote_provider)
+                    controller = PosixRemoteConfig(name=options.remote, provider=options.remote_provider, arch=options.remote_arch)
                     targets = controller_targets(mode, options, controller)
             else:
                 context, reason = f'--remote {options.remote}', FallbackReason.ENVIRONMENT
                 controller = None
-                targets = [PosixRemoteConfig(name=options.remote, python=native_python(options), provider=options.remote_provider)]
+                targets = [PosixRemoteConfig(name=options.remote, python=native_python(options), provider=options.remote_provider, arch=options.remote_arch)]
         elif mode == TargetMode.SHELL and options.remote.startswith('windows/'):
             if options.python and options.python not in CONTROLLER_PYTHON_VERSIONS:
                 raise ControllerNotSupportedError(f'--python {options.python}')
 
             controller = OriginConfig(python=native_python(options))
-            targets = [WindowsRemoteConfig(name=options.remote, provider=options.remote_provider)]
+            targets = [WindowsRemoteConfig(name=options.remote, provider=options.remote_provider, arch=options.remote_arch)]
         else:
             if not options.python:
                 raise PythonVersionUnspecifiedError(f'--remote {options.remote}')
 
             if controller_python(options.python):
-                controller = PosixRemoteConfig(name=options.remote, python=native_python(options), provider=options.remote_provider)
+                controller = PosixRemoteConfig(name=options.remote, python=native_python(options), provider=options.remote_provider, arch=options.remote_arch)
                 targets = controller_targets(mode, options, controller)
             else:
                 context, reason = f'--remote {options.remote} --python {options.python}', FallbackReason.PYTHON
                 controller = None
-                targets = [PosixRemoteConfig(name=options.remote, python=native_python(options), provider=options.remote_provider)]
+                targets = [PosixRemoteConfig(name=options.remote, python=native_python(options), provider=options.remote_provider, arch=options.remote_arch)]
 
         if not controller:
             if docker_available():
@@ -436,29 +460,32 @@ def get_legacy_host_config(
 
 
 def handle_non_posix_targets(
-    mode,  # type: TargetMode
-    options,  # type: LegacyHostOptions
-    targets,  # type: t.List[HostConfig]
-):  # type: (...) -> t.List[HostConfig]
+    mode: TargetMode,
+    options: LegacyHostOptions,
+    targets: list[HostConfig],
+) -> list[HostConfig]:
     """Return a list of non-POSIX targets if the target mode is non-POSIX."""
     if mode == TargetMode.WINDOWS_INTEGRATION:
         if options.windows:
-            targets = [WindowsRemoteConfig(name=f'windows/{version}', provider=options.remote_provider) for version in options.windows]
+            targets = [WindowsRemoteConfig(name=f'windows/{version}', provider=options.remote_provider, arch=options.remote_arch)
+                       for version in options.windows]
         else:
             targets = [WindowsInventoryConfig(path=options.inventory)]
     elif mode == TargetMode.NETWORK_INTEGRATION:
         if options.platform:
-            targets = [NetworkRemoteConfig(name=platform, provider=options.remote_provider) for platform in options.platform]
+            network_targets = [NetworkRemoteConfig(name=platform, provider=options.remote_provider, arch=options.remote_arch) for platform in options.platform]
 
             for platform, collection in options.platform_collection or []:
-                for entry in targets:
+                for entry in network_targets:
                     if entry.platform == platform:
                         entry.collection = collection
 
             for platform, connection in options.platform_connection or []:
-                for entry in targets:
+                for entry in network_targets:
                     if entry.platform == platform:
                         entry.connection = connection
+
+            targets = t.cast(list[HostConfig], network_targets)
         else:
             targets = [NetworkInventoryConfig(path=options.inventory)]
 
@@ -466,16 +493,18 @@ def handle_non_posix_targets(
 
 
 def default_targets(
-    mode,  # type: TargetMode
-    controller,  # type: ControllerHostConfig
-):  # type: (...) -> t.List[HostConfig]
+    mode: TargetMode,
+    controller: ControllerHostConfig,
+) -> list[HostConfig]:
     """Return a list of default targets for the given target mode."""
+    targets: list[HostConfig]
+
     if mode == TargetMode.WINDOWS_INTEGRATION:
         targets = [WindowsInventoryConfig(path=os.path.abspath(os.path.join(data_context().content.integration_path, 'inventory.winrm')))]
     elif mode == TargetMode.NETWORK_INTEGRATION:
         targets = [NetworkInventoryConfig(path=os.path.abspath(os.path.join(data_context().content.integration_path, 'inventory.networking')))]
     elif mode.multiple_pythons:
-        targets = controller.get_default_targets(HostContext(controller_config=controller))
+        targets = t.cast(list[HostConfig], controller.get_default_targets(HostContext(controller_config=controller)))
     else:
         targets = [ControllerConfig()]
 
