@@ -1,13 +1,14 @@
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+"""Disallow use of the dict.itervalues function."""
+from __future__ import annotations
 
 import re
 import sys
 
 
 def main():
+    """Main entry point."""
     for path in sys.argv[1:] or sys.stdin.read().splitlines():
-        with open(path, 'r') as path_fd:
+        with open(path, 'r', encoding='utf-8') as path_fd:
             for line, text in enumerate(path_fd.readlines()):
                 match = re.search(r'(?<! six)\.(itervalues)', text)
 
