@@ -5,21 +5,10 @@
 from __future__ import absolute_import, division, print_function
 __metaclass__ = type
 
-import pytest
 
 from ansible.module_utils.common.parameters import _list_deprecations
 
 
-@pytest.fixture
-def params():
-    return {
-        'name': 'bob',
-        'dest': '/etc/hosts',
-        'state': 'present',
-        'value': 5,
-    }
-
-
 def test_list_deprecations():
     argument_spec = {
         'old': {'type': 'str', 'removed_in_version': '2.5'},
